@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2009Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -23,9 +23,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 #ifndef STEPS_SOLVER_API_HPP
@@ -111,24 +111,39 @@ public:
     /// Run the solver for a step.
     virtual void step(void);
 
-    /// Set DT of the solver.
+    /// Set DT of the solver. Included for backwards compatibility but
+    /// replaced by setRk4DT
     ///
     /// \param dt Dt.
     virtual void setDT(double dt);
-    
+
+    /// Set DT of the numerical solver.
+    ///
+    /// \param dt Dt.
+    virtual void setRk4DT(double dt);
+
     virtual void setNSteps(uint nsteps);
-    
+
     virtual void setTime(double time);
 
+    /// Return the DT of the numerical solver
+    virtual double getRk4DT(void) const;
 
     /// Return the DT
-
+    /// Replaced by getRk4DT
     virtual double getDT(void) const;
+
     /// Return the A0
     virtual double getA0(void) const;
     /// Return the number of steps.
     virtual uint getNSteps(void) const;
 
+    /// checkpoint simulator state to a file
+    virtual void checkpoint(std::string const & file_name);
+
+    /// restore simulator state from a file
+    virtual void restore(std::string const & file_name);
+
     ////////////////////////////////////////////////////////////////////////
     // SOLVER STATE ACCESS:
     //      COMPARTMENT
@@ -340,16 +355,23 @@ public:
     /// \param vol Volume of the tetrahedron.
     void setTetVol(uint tidx, double vol);
 
+    /// Returns whether species s is defined in a tetrahedral volume
+    /// element (voxel).
+    ///
+    /// \param tidx Index of the tetrahedron.
+    /// \param s Name of the species.
+    bool getTetSpecDefined(uint tidx, std::string const & s) const;
+
     /// Returns the number of molecules of species s in a tetrahedral volume
     /// element (voxel).
     ///
-    /// \parem tidx Index of the tetrahedron.
+    /// \param tidx Index of the tetrahedron.
     /// \param s Name of the species.
     double getTetCount(uint tidx, std::string const & s) const;
 
     /// Sets the number of molecules of species s in a voxel.
     ///
-    /// \parem tidx Index of the tetrahedron.
+    /// \param tidx Index of the tetrahedron.
     /// \param s Name of the species.
     /// \param n Number of molecules of the species.
     void setTetCount(uint tidx, std::string const & s, double n);
@@ -632,6 +654,24 @@ public:
     /// \param r Name of the reaction.
     void resetPatchSReacExtent(std::string const & p, std::string const & r);
 
+    ////////////////////////////////////////////////////////////////////////
+    // SOLVER STATE ACCESS:
+    //      DIFFUSION BOUNDARIES
+    ////////////////////////////////////////////////////////////////////////
+
+    /// Activate or inactivate diffusion across a diffusion boundary for a species.
+    ///
+    /// \param db Name of the diffusion boundary.
+    /// \param s Name of the species.
+    /// \param act Bool to activate (true) or inactivate (false) diffusion.
+    void setDiffBoundaryDiffusionActive(std::string const & db, std::string const & s, bool act);
+
+    /// Returns whether diffusion is active across a diffusion boundary for a species.
+    ///
+    /// \param db Name of the diffusion boundary.
+    /// \param s Name of the species.
+    bool getDiffBoundaryDiffusionActive(std::string const & db, std::string const & s) const;
+
     ////////////////////////////////////////////////////////////////////////
     // SOLVER STATE ACCESS:
     //      TRIANGULAR SURFACE ELEMENTS
@@ -648,6 +688,12 @@ public:
     /// \param area Area of teh triangle.
     void setTriArea(uint tidx, double area);
 
+    /// Returns whether species s is defined in a triangle
+    ///
+    /// \param tidx Index of the triangle.
+    /// \param s Name of the species.
+    bool getTriSpecDefined(uint tidx, std::string const & s) const;
+
     /// Returns the number of molecules of species s in a triangle.
     ///
     /// \param tidx Index of the triangle.
@@ -793,6 +839,8 @@ protected:
     virtual double _getTetVol(uint tidx) const;
     virtual void _setTetVol(uint tidx, double vol);
 
+    virtual bool _getTetSpecDefined(uint tidx, uint sidx) const;
+
     virtual double _getTetCount(uint tidx, uint sidx) const;
     virtual void _setTetCount(uint tidx, uint sidx, double n);
 
@@ -857,6 +905,14 @@ protected:
     virtual uint _getPatchSReacExtent(uint pidx, uint ridx) const;
     virtual void _resetPatchSReacExtent(uint pidx, uint ridx);
 
+    ////////////////////////////////////////////////////////////////////////
+    // SOLVER STATE ACCESS:
+    //      DIFFUSION BOUNDARIES
+    ////////////////////////////////////////////////////////////////////////
+
+    virtual void _setDiffBoundaryDiffusionActive(uint dbidx, uint sidx, bool act);
+    virtual bool _getDiffBoundaryDiffusionActive(uint dbidx, uint sidx) const;
+
     ////////////////////////////////////////////////////////////////////////
     // SOLVER STATE ACCESS:
     //      TRIANGULAR SURFACE ELEMENTS
@@ -865,6 +921,8 @@ protected:
     virtual double _getTriArea(uint tidx) const;
     virtual void _setTriArea(uint tidx, double area);
 
+    virtual bool _getTriSpecDefined(uint tidx, uint sidx) const;
+
     virtual double _getTriCount(uint tidx, uint sidx) const;
     virtual void _setTriCount(uint tidx, uint sidx, double n);
 
