@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2009Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -23,9 +23,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 // STL headers.
@@ -94,6 +94,31 @@ void API::setTriArea(uint tidx, double area)
 
 ////////////////////////////////////////////////////////////////////////////////
 
+bool API::getTriSpecDefined(uint tidx, string const & s) const
+{
+	if (steps::tetmesh::Tetmesh * mesh = dynamic_cast<steps::tetmesh::Tetmesh*>(geom()))
+	{
+		if (tidx >= mesh->countTris())
+		{
+			std::ostringstream os;
+			os << "Triangle index out of range.";
+			throw steps::ArgErr(os.str());
+		}
+		// the following may throw exception if string is unknown
+		uint sidx = pStatedef->getSpecIdx(s);
+
+		return _getTriSpecDefined(tidx, sidx);
+	}
+	else
+	{
+		std::ostringstream os;
+		os << "Method not available for this solver.";
+		throw steps::NotImplErr();
+	}
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
 double API::getTriCount(uint tidx, string const & s) const
 {
 	if (steps::tetmesh::Tetmesh * mesh = dynamic_cast<steps::tetmesh::Tetmesh*>(geom()))
@@ -465,6 +490,13 @@ void API::_setTriArea(uint tidx, double area)
 
 ////////////////////////////////////////////////////////////////////////////////
 
+bool API::_getTriSpecDefined(uint tidx, uint sidx) const
+{
+	throw steps::NotImplErr();
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
 double API::_getTriCount(uint tidx, uint sidx) const
 {
     throw steps::NotImplErr();
