@@ -24,8 +24,8 @@
 
 
 /*
- *  Last Changed Rev:  $Rev: 422 $
- *  Last Changed Date: $Date: 2011-05-31 16:17:06 +0900 (Tue, 31 May 2011) $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
  *  Last Changed By:   $Author: iain $
  */
 
@@ -188,9 +188,6 @@ void stex::Tet::setDiffBndDirection(uint i)
 
 void stex::Tet::setNextTri(uint i, stex::Tri * t)
 {
-	assert (i <= 3);
-
-    // This is too common now to include this message- for any internal patch this happens
 	//if (pNextTet[i] != 0) std::cout << "WARNING: writing over nextTet index " << i;
     pNextTet[i] = 0;
     pNextTri[i] = t;
