@@ -24,8 +24,8 @@
 
 
 /*
- *  Last Changed Rev:  $Rev: 422 $
- *  Last Changed Date: $Date: 2011-05-31 16:17:06 +0900 (Tue, 31 May 2011) $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
  *  Last Changed By:   $Author: iain $
  */
 
@@ -39,6 +39,7 @@
 #include <limits>
 #include <queue>
 #include <fstream>
+#include <iomanip>
 
 // STEPS headers.
 #include "../common.h"
@@ -67,9 +68,6 @@
 
 ////////////////////////////////////////////////////////////////////////////////
 
-#define SCHEDULEWIDTH 16
-//#define MAXLEVELS 10
-
 ////////////////////////////////////////////////////////////////////////////////
 
 
@@ -111,12 +109,6 @@ stex::Tetexact::Tetexact(steps::model::Model * m, steps::wm::Geom * g, steps::rn
 , pTets()
 , pTris()
 , pA0(0.0)
-, pLevelSizes()
-, pLevels()
-, pBuilt(false)
-, pIndices(0)
-, pMaxUpSize(0)
-, pRannum(0)
 {
 	// Perform upcast.
 	pMesh = dynamic_cast<steps::tetmesh::Tetmesh*>(geom());
@@ -318,13 +310,14 @@ stex::Tetexact::Tetexact(steps::model::Model * m, steps::wm::Geom * g, steps::rn
     		// may have more than 1 neighbouring tri
     		// NOTE: The order here will end up being different to the
     		// neighbour order at the Tetmesh level
-
+            
 			// Now with diffusion boundaries, meaning tets can have neighbours that
 			// are in different comps, we must check the compartment
     		steps::tetexact::Tet * tet_in = pTets[tetinner];
-
+            
 			if (tet_in->nextTet(i) != 0 && tet_in->compdef() == tet_in->nextTet(i)->compdef()) continue;
 			if (tet_in->nextTri(i) != 0) continue;
+            
 			tet_in->setNextTri(i, pTris[t]);
     		break;
     	}
@@ -341,14 +334,14 @@ stex::Tetexact::Tetexact(steps::model::Model * m, steps::wm::Geom * g, steps::rn
     			for (uint i=0; i <= 4; ++i)
     			{
     				assert (i < 4);
-
     				// See above in that tets now store tets from different comps
     				steps::tetexact::Tet * tet_out = pTets[tetouter];
-
+                    
     				if (tet_out->nextTet(i) != 0 && tet_out->compdef() == tet_out->nextTet(i)->compdef()) continue;
-
+                    
     				if (tet_out->nextTri(i) != 0) continue;
-    				tet_out->setNextTri(i, pTris[t]);
+                    
+                    tet_out->setNextTri(i, pTris[t]);
     				break;
 				}
     		}
@@ -472,7 +465,9 @@ stex::Tetexact::~Tetexact(void)
     for (CompPVecCI c = pComps.begin(); c != comp_e; ++c) delete *c;
     PatchPVecCI patch_e = pPatches.end();
     for (PatchPVecCI p = pPatches.begin(); p != patch_e; ++p) delete *p;
-
+    DiffBoundaryPVecCI db_e = pDiffBoundaries.end();
+    for (DiffBoundaryPVecCI db = pDiffBoundaries.begin(); db!=db_e; ++db) delete *db;
+    
     TetPVecCI tet_e = pTets.end();
     for (TetPVecCI t = pTets.begin(); t != tet_e; ++t)
     {
@@ -484,11 +479,19 @@ stex::Tetexact::~Tetexact(void)
     {
         if ((*t) != 0) delete (*t);
     }
-
-    std::for_each(pLevels.begin(), pLevels.end(), DeleteArray());
-
-    delete[] pIndices;
-    delete[] pRannum;
+        
+    uint ngroups = nGroups.size();
+    for (uint i = 0; i < ngroups; i++) {
+        free(nGroups[i]->indices);
+        delete nGroups[i];
+    }
+        
+    ngroups = pGroups.size();
+    for (uint i = 0; i < ngroups; i++) {
+        free(pGroups[i]->indices);
+        delete pGroups[i];
+    }
+    
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -598,83 +601,7 @@ std::string stex::Tetexact::getSolverEmail(void) const
 ////////////////////////////////////////////////////////////////////////////////
 
 void stex::Tetexact::_setup(void)
-{
-    // geometry based optimization
-    // nearby KProcs get picked first
-    // this is to minimize update branches
-    std::queue<uint> append_queue;
-
-    std::set<uint> tets_set;
-    std::set<uint> tris_set;
-
-    std::set<uint>::iterator it;
-
-    uint ntets = pTets.size();
-    uint ntris = pTris.size();
-
-    for (uint t = 0; t < ntets; t++) {
-        if (pTets[t] != 0) tets_set.insert(t);
-    }
-
-    for (uint t = 0; t < ntris; t++) {
-        if (pTris[t] != 0) tris_set.insert(t);
-    }
-
-
-    while (!tets_set.empty() || !tris_set.empty() || !append_queue.empty()) {
-        while (!append_queue.empty()) {
-            uint idx = append_queue.front();
-            append_queue.pop();
-
-            // if it is a tri
-            if (idx >= ntets) {
-                pTris[idx - ntets]->setupKProcs(this);
-            }
-            // if it is a tet
-            else {
-                pTets[idx]->setupKProcs(this);
-                steps::tetexact::Tet * t = pTets[idx];
-
-                for (uint i = 0; i < 4; i++) {
-                    steps::tetexact::Tri * neib_tri = t->nextTri(i);
-                    if (neib_tri == 0) continue;
-                    uint ntri_idx = neib_tri->idx();
-                    if (tris_set.find(ntri_idx) != tris_set.end()) {
-                        append_queue.push(ntri_idx + ntets);
-                        tris_set.erase(ntri_idx);
-                    }
-
-                    steps::tetexact::Tet * neib_tet = t->nextTet(i);
-                    if (neib_tet == 0) continue;
-                    uint ntet_idx = neib_tet->idx();
-                    if (tets_set.find(ntet_idx) != tets_set.end()) {
-                        append_queue.push(ntet_idx);
-                        tets_set.erase(ntet_idx);
-                    }
-                }
-            }
-        }
-
-
-        if (!tets_set.empty()) {
-            append_queue.push(*(tets_set.begin()));
-            tets_set.erase(tets_set.begin());
-        }
-        else if (!tris_set.empty()) {
-            append_queue.push(*(tets_set.begin()) + ntets);
-            tris_set.erase(tris_set.begin());
-        }
-    }
-
-    assert(tets_set.empty() && tris_set.empty());
-
-	// Resolve all dependencies
-    TetPVecCI tet_end = pTets.end();
-    TriPVecCI tri_end = pTris.end();
-
-    /*  old version, insert KProcs without optimization
-
-
+{   
 	TetPVecCI tet_end = pTets.end();
 	for (TetPVecCI t = pTets.begin(); t != tet_end; ++t)
 	{
@@ -694,10 +621,7 @@ void stex::Tetexact::_setup(void)
 
 		(*t)->setupKProcs(this);
 	}
-
-
-    */
-
+    
 	for (TetPVecCI t = pTets.begin(); t != tet_end; ++t)
 	{
 		// DEBUG: vector holds all possible tetrahedrons,
@@ -710,6 +634,8 @@ void stex::Tetexact::_setup(void)
 		    (*k)->setupDeps();
 		}
 	}
+    
+    
 	for (TriPVecCI t = pTris.begin(); t != tri_end; ++t)
 	{
 		// DEBUG: vector holds all possible triangles, but
@@ -723,7 +649,7 @@ void stex::Tetexact::_setup(void)
 	    }
 	}
 
-	_build();
+	nEntries = pKProcs.size();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -820,10 +746,26 @@ void stex::Tetexact::reset(void)
 		(*t)->reset();
 	}
 
+    uint ngroups = nGroups.size();
+    for (uint i = 0; i < ngroups; i++) {
+        free(nGroups[i]->indices);
+        delete nGroups[i];
+    }
+    nGroups.clear();
+    
+    ngroups = pGroups.size();
+    for (uint i = 0; i < ngroups; i++) {
+        free(pGroups[i]->indices);
+        delete pGroups[i];
+    }
+    pGroups.clear();
+    
+    pSum = 0.0;
+    nSum = 0.0;
+    pA0 = 0.0;
+    
 	statedef()->resetTime();
 	statedef()->resetNSteps();
-
-	_reset();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -838,13 +780,16 @@ void stex::Tetexact::run(double endtime)
 	}
 	while (statedef()->time() < endtime)
 	{
+        //std::cout << "pass1\n";
 		stex::KProc * kp = _getNext();
 		if (kp == 0) break;
 		double a0 = getA0();
 		if (a0 == 0.0) break;
 		double dt = rng()->getExp(a0);
 		if ((statedef()->time() + dt) > endtime) break;
+        //std::cout << "pass2\n";
 		_executeStep(kp, dt);
+        //std::cout << "pass3\n";
 	}
 	statedef()->setTime(endtime);
 }
@@ -864,6 +809,21 @@ void stex::Tetexact::advance(double adv)
 	run(endtime);
 }
 
+////////////////////////////////////////////////////////////////////////
+
+void stex::Tetexact::advanceSteps(uint nsteps)
+{
+	while (nsteps != 0) {
+        stex::KProc * kp = _getNext();
+        if (kp == 0) return;
+        double a0 = getA0();
+        if (a0 == 0.0) return;
+        double dt = rng()->getExp(a0);
+        _executeStep(kp, dt);
+        nsteps--;
+    }
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 
 void stex::Tetexact::step(void)
@@ -1795,332 +1755,173 @@ void stex::Tetexact::addKProc(steps::tetexact::KProc * kp)
 
 void stex::Tetexact::_build(void)
 {
-	assert (pBuilt == false);
-
-    // Setup level.
-    uint clsize = pKProcs.size();
-    if (clsize == 0) return;
-
-    // Work up.
-    uint clevel = 0;
-    do
-    {
-        // Make sure the new size is a multiple of SCHEDULEWIDTH.
-        uint extra = clsize % SCHEDULEWIDTH;
-        if (extra != 0) clsize += SCHEDULEWIDTH - extra;
-
-        // Create the level and add it.
-        double * level = new double[clsize];
-        std::fill_n(level, clsize, 0.0);
-        pLevelSizes.push_back(clsize);
-        pLevels.push_back(level);
-
-        // Prepare for next level.
-        clevel++;
-        clsize = clsize / SCHEDULEWIDTH;
-    }
-    while (clsize > 1);
-
-    // Set top level.
-    pA0 = 0.0;
-
-    // Time to create ONE indices table to hold the run's present reaction of
-    // choice's update vector. This will be re-used and replace old-version's
-    // hard-coded table in _update. Size is the maximum possible, found by looping
-    // over all KProcs. This little bit of computational time is well worth all
-    // that needless memory allocation
-	uint maxupdvecsize = 0;
-	TetPVecCI tet_end = pTets.end();
-	for (TetPVecCI tet = pTets.begin(); tet != tet_end; ++tet)
-	{
-		// DEBUG: pTets element may be zero pointer if tet not added to a compartment
-		if ((*tet) == 0) continue;
-
-		// To find the maximum size of an individual kproc's upd vec size
-		uint kpuv_size = 0;
-		SchedIDXSet updset;
-		KProcPVecCI kproc_end = (*tet)->kprocEnd();
-		for (KProcPVecCI k = (*tet)->kprocBegin(); k != kproc_end; ++k)
-		{
-			updset.insert((*k)->schedIDX());
-			if ((*k)->updVecSize() > kpuv_size) kpuv_size = (*k)->updVecSize();
-		}
-
-	    // Loop over neighbouring triangles.
-		for (uint i = 0; i < 4; ++i)
-		{
-			Tri * tri = (*tet)->nextTri(i);
-			if (tri == 0) continue;
-			kproc_end = tri->kprocEnd();
-			for (KProcPVecCI k = tri->kprocBegin(); k != kproc_end; ++k)
-			{
-				updset.insert((*k)->schedIDX());
-			}
-		}
-		SchedIDXVec updvec;
-		schedIDXSet_To_Vec(updset, updvec);
-		if (maxupdvecsize < updvec.size()) maxupdvecsize = updvec.size();
-		if (maxupdvecsize < kpuv_size) maxupdvecsize = kpuv_size;
-	}
-	TriPVecCI tri_end = pTris.end();
-	for (TriPVecCI tri = pTris.begin(); tri != tri_end; ++tri)
-	{
-		uint kpuv_size = 0;
-		//DEBUG: Didn't allow for a zero pointer (not added to geometry i.e. not
-		// belonging to a patch
-		if ((*tri) == 0) continue;
-
-		SchedIDXSet updset;
-	    KProcPVecCI kproc_end = (*tri)->kprocEnd();
-		for (KProcPVecCI k = (*tri)->kprocBegin(); k != kproc_end; ++k)
-		{
-			updset.insert((*k)->schedIDX());
-			if ((*k)->updVecSize() > kpuv_size) kpuv_size = (*k)->updVecSize();
-		}
-		SchedIDXVec updvec;
-		schedIDXSet_To_Vec(updset, updvec);
-		if (maxupdvecsize < updvec.size()) maxupdvecsize = updvec.size();
-		if (maxupdvecsize < kpuv_size) maxupdvecsize = kpuv_size;
-	}
-
-	pMaxUpSize = maxupdvecsize;
-	pIndices = new uint[pMaxUpSize];
-
-    // Also let's create a random number holder-table,
-    // size of number of KProcs % SCHEDULEWIDTH or pLevels.size()
-    // This will be re-used in _getNext as apposed to hard-coded (again maximum
-    // limit).
-    uint lsize = pLevels.size();
-    pRannum = new double[lsize];
-
-    pBuilt = true;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
 steps::tetexact::KProc * stex::Tetexact::_getNext(void) const
 {
+    #ifdef SSA_DEBUG
+    std::cout << "SSA: Search for next event\n";
+    #endif
     assert(pA0 >= 0.0);
     // Quick check to see whether nothing is there.
-    if (pA0 == 0.0) return 0;
+    if (pA0 == 0.0) return NULL;
+    
+    double selector = pA0 * rng()->getUnfII();
+    
+    #ifdef SSA_DEBUG
+    std::cout << "selector: " << selector << " in total " << pA0 << "\n";
+    #endif
+    
+    double partial_sum = 0.0;
+    
+    uint n_neg_groups = nGroups.size();
+    uint n_pos_groups = pGroups.size();
+    
+    for (uint i = 0; i < n_neg_groups; i++) {
+        CRGroup* group = nGroups[i];
+        if (group->size == 0) continue;
 
-    // Start at top level.
-    uint clevel = pLevels.size();
-    // And start at the first node of that level.
-    uint cur_node = 0;
-
-    // Prepare random numbers.
-    for (uint i = 0; i < clevel; ++i)
-    {
-        pRannum[i] = rng()->getUnfIE();
-    }
-
-    // Run until top level.
-    double a0 = pA0;
-    double * level = 0;
-    while (clevel != 0)
-    {
-        // Decrease the current level.
-        clevel--;
-        // and start looking in the right place.
-        cur_node *= SCHEDULEWIDTH;
-        uint max_node = cur_node + SCHEDULEWIDTH;
-
-        // Fetch the level.
-        level = pLevels[clevel];
-
-        // Compute local selector.
-        double selector = pRannum[clevel] * a0;
-
-        // Compare.
-        double accum = 0.0;
-        // 27/10/09 I.H. 'old' removed from for loop because not used.
-        // double old = 0.0;
-        double curval = 0.0;
-        for (uint i = 0; i < SCHEDULEWIDTH; ++i)
-        {
-            curval = level[cur_node];
-            if (selector < curval + accum) break;
-            accum += curval;
-            // old = accum;
-            cur_node++;
+        if (selector > partial_sum + group->sum) {
+            partial_sum += group->sum;
+            #ifdef SSA_DEBUG
+            std::cout << "increase partial sum to " << partial_sum;
+            std::cout << " by neg group " << i << "\n";
+            #endif            
+            continue;
         }
-
-        // Checks.
-        assert(cur_node < max_node);
-        assert(curval > 0.0);
-        a0 = curval;
+        
+        double g_max = group->max;
+        double random_rate = g_max * rng()->getUnfII();;
+        uint group_size = group->size;
+        uint random_pos = rng()->get() % group_size;
+        KProc* random_kp = group->indices[random_pos];
+        
+        #ifdef SSA_DEBUG
+        std::cout << "search for event\n";
+        std::cout << "group max: " << g_max << "\n";
+        std::cout << "random rate: " << random_rate << "\n";
+        std::cout << "random pos " << random_pos << "\n";
+        std::cout << "random kp rate " << random_kp->crData.rate << "\n";
+        #endif
+        
+        while (random_kp->crData.rate <= random_rate) {
+            random_rate = g_max * rng()->getUnfII();
+            random_pos = rng()->get() % group_size;
+            random_kp = group->indices[random_pos];
+            #ifdef SSA_DEBUG
+            std::cout << "renew search\n";
+            std::cout << "random rate: " << random_rate << "\n";
+            std::cout << "random pos " << random_pos << "\n";
+            std::cout << "random kp rate " << random_kp->crData.rate << "\n";
+            #endif
+        }
+        
+        #ifdef SSA_DEBUG
+        std::cout << "selected kp index: " << random_kp->schedIDX() << "\n";
+        std::cout << "--------------------------------------------------------\n";
+        #endif
+        return random_kp;        
     }
-
-    // Check.
-    assert(cur_node < pKProcs.size());
-    return pKProcs[cur_node];
+    
+    for (uint i = 0; i < n_pos_groups; i++) {
+        CRGroup* group = pGroups[i];
+        if (group->size == 0) continue;
+        
+        if (selector > partial_sum + group->sum) {
+            partial_sum += group->sum;
+            #ifdef SSA_DEBUG
+            std::cout << "increase partial sum to " << partial_sum;
+            std::cout << " by neg group " << i << "\n";
+            #endif            
+            continue;
+        }
+        
+        double g_max = group->max;
+        double random_rate = g_max * rng()->getUnfII();;
+        uint group_size = group->size;
+        uint random_pos = rng()->get() % group_size;
+        KProc* random_kp = group->indices[random_pos];
+        
+        #ifdef SSA_DEBUG
+        std::cout << "search for event\n";
+        std::cout << "group max: " << g_max << "\n";
+        std::cout << "random rate: " << random_rate << "\n";
+        std::cout << "random pos " << random_pos << "\n";
+        std::cout << "random kp rate " << random_kp->crData.rate << "\n";
+        #endif
+        
+        while (random_kp->crData.rate <= random_rate) {
+            random_rate = g_max * rng()->getUnfII();
+            random_pos = rng()->get() % group_size;
+            random_kp = group->indices[random_pos];
+            #ifdef SSA_DEBUG
+            std::cout << "renew search\n";
+            std::cout << "random rate: " << random_rate << "\n";
+            std::cout << "random pos " << random_pos << "\n";
+            std::cout << "random kp rate " << random_kp->crData.rate << "\n";
+            #endif
+        }
+        
+        #ifdef SSA_DEBUG
+        std::cout << "selected kp index: " << random_kp->schedIDX() << "\n";
+        std::cout << "--------------------------------------------------------\n";
+        #endif
+        return random_kp;        
+    }    
+    
+        
+    std::cerr << "Cannot find any suitable entry.\n";
+    std::cerr << "A0: " << std::setprecision (15) << pA0 << "\n";
+    std::cerr << "Selector: " << std::setprecision (15) << selector << "\n";
+    std::cerr << "Current Partial Sum: " << std::setprecision (15) << partial_sum << "\n";
+    
+    std::cerr << "Distribution of group sums\n";
+    std::cerr << "Negative groups\n";
+    
+    for (uint i = 0; i < n_neg_groups; i++) {
+        std::cerr << i << ": " << std::setprecision (15) << nGroups[i]->sum << "\n"; 
+    }
+    std::cerr << "Positive groups\n";
+    for (uint i = 0; i < n_pos_groups; i++) {
+        std::cerr << i << ": " << std::setprecision (15) << pGroups[i]->sum << "\n"; 
+    }
+    
+    throw;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
 void stex::Tetexact::_reset(void)
 {
-    if (pKProcs.size() == 0) return;
-
-    // Reset the basic level: compute rates.
-    double * oldlevel = pLevels[0];
-    uint cur_node = 0;
-    std::vector<KProc*>::iterator kp_end = pKProcs.end();
-    for (std::vector<KProc*>::iterator kp = pKProcs.begin(); kp != kp_end; ++kp)
-    {
-        oldlevel[cur_node++] = (*kp)->rate();
-    }
-
-    // Work up.
-    for (uint cur_level = 1; cur_level < pLevels.size(); ++cur_level)
-    {
-        // Compute the number of nodes to reset on this level.
-        uint numnodes = pLevelSizes[cur_level - 1] / SCHEDULEWIDTH;
-
-        // Fetch a pointer to this level.
-        double * level = pLevels[cur_level];
-
-        // Recompute them.
-        uint child_node = 0;
-        for (cur_node = 0; cur_node < numnodes; ++cur_node)
-        {
-            double val = 0.0;
-            for (uint i = 0; i < SCHEDULEWIDTH; ++i)
-            {
-                val += oldlevel[child_node++];
-            }
-            level[cur_node] = val;
-        }
-
-        // Copy the level.
-        oldlevel = level;
-    }
-
-    // Compute zero propensity.
-    pA0 = 0.0;
-    for (uint i = 0; i < SCHEDULEWIDTH; ++i)
-    {
-        pA0 += oldlevel[i];
-    }
+    _update();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
 void stex::Tetexact::_executeStep(steps::tetexact::KProc * kp, double dt)
 {
-	SchedIDXVec const & upd = kp->apply(rng());
-	_update(upd);
-	statedef()->incTime(dt);
-	statedef()->incNSteps(1);
-}
-
-////////////////////////////////////////////////////////////////////////////////
-
-void stex::Tetexact::_update(SchedIDXVec const & entries)
-{
-    if (countKProcs() == 0) return;
-
-    // Prefetch zero level.
-    double * level0 = pLevels[0];
-    // Number of entries.
-    assert(entries.size() <= pMaxUpSize);								/////////
-
-    // Recompute rates.
-    SchedIDXVecCI sidx_end = entries.end();
-    uint prev_e = 0xFFFFFFFF;
-    uint cur_e = 0;
-    for (SchedIDXVecCI sidx = entries.begin(); sidx != sidx_end; ++sidx)
-    {
-        // Fetch index.
-        uint idx = *sidx;
-        // Recompute rate, get difference, and store.
-        double newrate = pKProcs[idx]->rate();
-        level0[idx] = newrate;
-
-        // Store and collapse if possible.
-        idx /= SCHEDULEWIDTH;
-        if (prev_e == 0xFFFFFFFF)
-        {
-            prev_e = 0;
-            pIndices[cur_e++] = idx;
-        }
-        else if (pIndices[prev_e] != idx)
-        {
-            prev_e = cur_e;
-            pIndices[cur_e++] = idx;
-        }
-    }
-    uint nentries = cur_e;
-
-    // Update upper levels.
-    uint nlevels = pLevels.size();
-    double * prevlevel = pLevels[0];
-    for (uint l = 1; l < nlevels; ++l)
-    {
-        // Update the first entry.
-        cur_e = 0;
-        prev_e = 0xFFFFFFFF;
-
-        // Fetch a pointer to the current level.
-        double * currlevel = pLevels[l];
-
-        // Recompute the entries.
-        for (uint e = 0; e < nentries; ++e)
-        {
-            // Fetch index.
-            uint idx = pIndices[e];
-
-            // Recompute.
-            double val = 0.0;
-            uint idx2 = idx * SCHEDULEWIDTH;
-            for (uint i = 0; i < SCHEDULEWIDTH; ++i)
-            {
-                val += prevlevel[idx2++];
-            }
-            currlevel[idx] = val;
-
-            // Store and collapse if possible.
-            idx /= SCHEDULEWIDTH;
-            if (prev_e == 0xFFFFFFFF)
-            {
-                prev_e = 0;
-                pIndices[cur_e++] = idx;
-            }
-            else if (pIndices[prev_e] != idx)
-            {
-                prev_e = cur_e;
-                pIndices[cur_e++] = idx;
-            }
-        }
-
-        // Update the pointer to the previous level.
-        prevlevel = currlevel;
-
-        // cur_e now is the new number of entries to handle.
-        nentries = cur_e;
-    }
-
-    // Update zero propensity.
-    double * toplevel = pLevels[pLevels.size() - 1];
-    pA0 = 0.0;
-    for (uint i = 0; i < SCHEDULEWIDTH; ++i)
-    {
-        pA0 += toplevel[i];
-    }
+    //std::cout << "passe1\n";
+    std::vector<KProc*> const & upd = kp->apply(rng());
+	//std::cout << "passe2\n";
+    _update(upd);
+    //std::cout << "passe3\n";
+    statedef()->incTime(dt);
+    statedef()->incNSteps();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
 void stex::Tetexact::_updateSpec(steps::tetexact::Tet * tet, uint spec_lidx)
 {
-    SchedIDXSet updset;
+    std::set<stex::KProc*> updset;
 
     // Loop over tet.
     KProcPVecCI kproc_end = tet->kprocEnd();
     for (KProcPVecCI k = tet->kprocBegin(); k != kproc_end; ++k)
     {
-        updset.insert((*k)->schedIDX());
+        updset.insert(*k);
     }
 
     // Loop over neighbouring triangles.
@@ -2131,14 +1932,13 @@ void stex::Tetexact::_updateSpec(steps::tetexact::Tet * tet, uint spec_lidx)
         kproc_end = tri->kprocEnd();
         for (KProcPVecCI k = tri->kprocBegin(); k != kproc_end; ++k)
         {
-            updset.insert((*k)->schedIDX());
+            updset.insert(*k);
         }
     }
 
     // Send the list of kprocs that need to be updated to the schedule.
     if (updset.empty()) return;
-    SchedIDXVec updvec;
-    schedIDXSet_To_Vec(updset, updvec);
+    std::vector<KProc*> updvec(updset.begin(), updset.end());
 
     _update(updvec);
 }
@@ -2147,18 +1947,18 @@ void stex::Tetexact::_updateSpec(steps::tetexact::Tet * tet, uint spec_lidx)
 
 void stex::Tetexact::_updateSpec(steps::tetexact::Tri * tri, uint spec_lidx)
 {
-    SchedIDXSet updset;
+    std::set<stex::KProc*> updset;
 
     KProcPVecCI kproc_end = tri->kprocEnd();
     for (KProcPVecCI k = tri->kprocBegin(); k != kproc_end; ++k)
     {
-        updset.insert((*k)->schedIDX());
+        updset.insert(*k);
     }
 
     // Send the list of kprocs that need to be updated to the schedule.
     if (updset.empty()) return;
-    SchedIDXVec updvec;
-    schedIDXSet_To_Vec(updset, updvec);
+    
+    std::vector<stex::KProc*> updvec(updset.begin(), updset.end());
 
     _update(updvec);
 }
@@ -2761,10 +2561,8 @@ void stex::Tetexact::_setTetReacK(uint tidx, uint ridx, double kf)
 
 	tet->reac(lridx)->setKcst(kf);
 
-	// Update the rates for all KProcs affected by this change
-	SchedIDXVec updvec;
-	updvec.push_back(tet->reac(lridx)->schedIDX());
-	_update(updvec);
+	_updateElement(tet->reac(lridx));
+    _updateSum();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -2821,9 +2619,8 @@ void stex::Tetexact::_setTetReacActive(uint tidx, uint ridx, bool act)
 
 	tet->reac(lridx)->setActive(act);
 
-	SchedIDXVec updvec;
-	updvec.push_back(tet->reac(lridx)->schedIDX());
-	_update(updvec);
+	_updateElement(tet->reac(lridx));
+    _updateSum();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -2880,10 +2677,8 @@ void stex::Tetexact::_setTetDiffD(uint tidx, uint didx, double dk)
 
 	tet->diff(ldidx)->setDcst(dk);
 
-	// Update the rates for all KProcs affected by this change
-	SchedIDXVec updvec;
-	updvec.push_back(tet->diff(ldidx)->schedIDX());
-	_update(updvec);
+	_updateElement(tet->diff(ldidx));
+    _updateSum();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -2940,9 +2735,8 @@ void stex::Tetexact::_setTetDiffActive(uint tidx, uint didx, bool act)
 
 	tet->diff(ldidx)->setActive(act);
 
-	SchedIDXVec updvec;
-	updvec.push_back(tet->diff(ldidx)->schedIDX());
-	_update(updvec);
+	_updateElement(tet->diff(ldidx));
+    _updateSum();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -3289,11 +3083,8 @@ void stex::Tetexact::_setTriSReacK(uint tidx, uint ridx, double kf)
 
 	tri->sreac(lsridx)->setKcst(kf);
 
-	// Update the rates for all KProcs affected by this change
-	SchedIDXVec updvec;
-	updvec.push_back(tri->sreac(lsridx)->schedIDX());
-	_update(updvec);
-
+	_updateElement(tri->sreac(lsridx));
+    _updateSum();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -3350,9 +3141,8 @@ void stex::Tetexact::_setTriSReacActive(uint tidx, uint ridx, bool act)
 
 	tri->sreac(lsridx)->setActive(act);
 
-	SchedIDXVec updvec;
-	updvec.push_back(tri->sreac(lsridx)->schedIDX());
-	_update(updvec);
+	_updateElement(tri->sreac(lsridx));
+    _updateSum();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -3438,4 +3228,200 @@ double stex::Tetexact::_getTriSReacA(uint tidx, uint ridx) const
 
 ////////////////////////////////////////////////////////////////////////////////
 
+void stex::Tetexact::_updateElement(KProc* kp)
+{
+#ifdef SSA_DEBUG
+    std::cout << "SSA: Update KProc element " << kp->schedIDX() << "\n";
+#endif
+    
+    double new_rate = kp->rate();
+    
+#ifdef SSA_DEBUG
+    std::cout << "new rate: " << new_rate << "\n";
+#endif
+    
+    CRKProcData & data = kp->crData;
+    double old_rate = data.rate;
+    
+    data.rate = new_rate;
+    
+#ifdef SSA_DEBUG
+    std::cout << "data recorded: " << (data.recorded ? "Yes" : "No") << "\n";
+    std::cout << "pow: " << data.pow << "\n";
+    std::cout << "pos: " << data.pos << "\n";
+    std::cout << "rate: " << data.rate << "\n";
+#endif
+    
+    if (old_rate == new_rate)  return;
+    
+    
+    // new rate in positive groups
+    if (new_rate >= 0.5) {
+#ifdef SSA_DEBUG
+        std::cout << "new rate in positive group\n";
+#endif
+        
+        // pow is the same
+        int old_pow = data.pow;
+        int new_pow;
+        double temp = frexp(new_rate, &new_pow);
+        
+#ifdef SSA_DEBUG
+        std::cout << "power changes from " << old_pow << " to " << new_pow << "\n";
+#endif
+        
+        if (old_pow == new_pow && data.recorded) {
+            
+            CRGroup* old_group = _getGroup(old_pow);
+            
+#ifdef SSA_DEBUG
+            std::cout << "in the same power group, change group sum and positive sum\n";
+            std::cout << "old group sum: " << old_group->sum << " old pSum " << pSum << "\n";
+#endif
+            
+            old_group->sum += (new_rate - old_rate);
+            
+#ifdef SSA_DEBUG
+            std::cout << "new group sum: " << old_group->sum << " new pSum " << pSum << "\n";
+#endif
+        }
+        // pow is not the same
+        else {
+            data.pow = new_pow;
+            
+            
+            if (data.recorded) {
+#ifdef SSA_DEBUG
+                std::cout << "remove data from group with power " << old_pow << "\n";
+#endif
+                
+                // remove old
+                CRGroup* old_group = _getGroup(old_pow);
+                (old_group->size) --;
+                
+                if (old_group->size == 0) old_group->sum = 0.0;
+                else {
+                    old_group->sum -= old_rate;
+                    
+                    KProc* last = old_group->indices[old_group->size];
+                    old_group->indices[data.pos] = last;
+                    last->crData.pos = data.pos;
+                }
+            }
+            
+            // add new
+#ifdef SSA_DEBUG
+            std::cout << "add data to group with power " << new_pow << "\n";
+#endif
+            if (pGroups.size() <= new_pow) _extendPGroups(new_pow + 1);
+            
+            CRGroup* new_group = pGroups[new_pow];
+            
+            assert(new_group != NULL);
+            if (new_group->size == new_group->capacity) _extendGroup(new_group);
+            uint pos = new_group->size;
+            new_group->indices[pos] = kp;
+            new_group->size++;
+            new_group->sum += new_rate;
+            data.pos = pos;
+            
+        }
+        data.recorded = true;
+        
+    }
+    // new rate in negative group
+    else if (new_rate < 0.5 && new_rate > 1e-20) {
+        int old_pow = data.pow;
+        int new_pow;
+        double temp = frexp(new_rate, &new_pow);
+        
+#ifdef SSA_DEBUG
+        std::cout << "power changes from " << old_pow << " to " << new_pow << "\n";
+#endif
+        
+        if (old_pow == new_pow && data.recorded) {
+            
+            CRGroup* old_group = _getGroup(old_pow);
+#ifdef SSA_DEBUG
+            std::cout << "in the same power group, change group sum and negative sum\n";
+            std::cout << "old group sum: " << old_group->sum << " old nSum " << nSum << "\n";
+#endif
+            
+            old_group->sum += (new_rate - old_rate);
+            
+#ifdef SSA_DEBUG
+            std::cout << "new group sum: " << old_group->sum << " new nSum " << nSum << "\n";
+#endif
+        }
+        // pow is not the same
+        else {
+            data.pow = new_pow;
+            
+            if (data.recorded) {
+#ifdef SSA_DEBUG
+                std::cout << "remove data from group with power " << old_pow << "\n";
+#endif
+                
+                CRGroup* old_group = _getGroup(old_pow);
+                (old_group->size) --;
+                
+                if (old_group->size == 0) old_group->sum = 0.0;
+                else {
+                    old_group->sum -= old_rate;
+                    
+                    KProc* last = old_group->indices[old_group->size];
+                    old_group->indices[data.pos] = last;
+                    last->crData.pos = data.pos;
+                }
+            }
+            
+            // add new
+#ifdef SSA_DEBUG
+            std::cout << "add data to group with power " << new_pow << "\n";
+#endif
+            
+            if (nGroups.size() <= -new_pow) _extendNGroups(-new_pow + 1);
+            
+            CRGroup* new_group = nGroups[-new_pow];
+            
+            if (new_group->size == new_group->capacity) _extendGroup(new_group);
+            uint pos = new_group->size;
+            new_group->indices[pos] = kp;
+            new_group->size++;
+            new_group->sum += new_rate;
+            data.pos = pos;
+            
+        }
+        data.recorded = true;
+    }
+    
+    else {
+        
+        if (data.recorded) {
+#ifdef SSA_DEBUG
+            std::cout << "remove data from group with power " << data.pow << "\n";
+#endif
+            CRGroup* old_group = _getGroup(data.pow);
+            
+            // remove old
+            old_group->size --;
+            
+            if (old_group->size == 0) old_group->sum = 0.0;
+            else {
+                old_group->sum -= old_rate;
+                
+                KProc* last = old_group->indices[old_group->size];
+                old_group->indices[data.pos] = last;
+                last->crData.pos = data.pos;
+            }
+        }
+        data.recorded = false;
+    }
+    
+#ifdef SSA_DEBUG
+    std::cout << "--------------------------------------------------------\n";
+#endif
+} 
+
+
 // END
