@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2009Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -24,9 +24,9 @@
 
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 // Standard library & STL headers.
@@ -48,16 +48,28 @@ NAMESPACE_ALIAS(steps::math, smath);
 
 ////////////////////////////////////////////////////////////////////////////////
 
-static inline double comp_ccst(double kcst, double vol, uint order)
+static inline double comp_ccst_vol(double kcst, double vol, uint order)
 {
     double vscale = 1.0e3 * vol * smath::AVOGADRO;
     int o1 = static_cast<int>(order) - 1;
-    if (o1 < 0) o1 = 0;
+    // I.H 5/1/2011 Removed this strange special behaviour for zero-order
+    // if (o1 < 0) o1 = 0;
     return kcst * pow(vscale, static_cast<double>(-o1));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
+static inline double comp_ccst_area(double kcst, double area, uint order)
+{
+    double ascale = area * smath::AVOGADRO;
+    int o1 = static_cast<int>(order) - 1;
+    // I.H 5/1/2011 Removed this strange special behaviour for zero-order
+    // if (o1 < 0) o1 = 0;
+    return kcst * pow(ascale, static_cast<double>(-o1));
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
 swmd::SReac::SReac(ssolver::SReacdef * srdef, swmd::Patch * patch)
 : KProc()
 , pSReacdef(srdef)
@@ -67,21 +79,33 @@ swmd::SReac::SReac(ssolver::SReacdef * srdef, swmd::Patch * patch)
 {
 	assert (pSReacdef != 0);
 	assert (pPatch != 0);
-	double vol;
-	if (defsr()->inside() == true)
+
+	uint lsridx = pPatch->def()->sreacG2L(defsr()->gidx());
+	double kcst = pPatch->def()->kcst(lsridx);
+
+	if (defsr()->surf_surf() == false)
 	{
-		assert(pPatch->iComp() != 0);
-		vol = pPatch->iComp()->def()->vol();
+		double vol;
+		if (defsr()->inside() == true)
+		{
+			assert(pPatch->iComp() != 0);
+			vol = pPatch->iComp()->def()->vol();
+		}
+		else
+		{
+			assert (pPatch->oComp() != 0);
+			vol = pPatch->oComp()->def()->vol();
+		}
+
+		pCcst = comp_ccst_vol(kcst, vol, defsr()->order());
 	}
 	else
 	{
-		assert (pPatch->oComp() != 0);
-		vol = pPatch->oComp()->def()->vol();
+		double area;
+		area = pPatch->def()->area();
+		pCcst = comp_ccst_area(kcst, area, defsr()->order());
 	}
 
-	uint lsridx = pPatch->def()->sreacG2L(pSReacdef->gidx());
-	double kcst = pPatch->def()->kcst(lsridx);
-	pCcst = comp_ccst(kcst, vol, pSReacdef->order());
 	assert (pCcst >= 0);
 }
 
@@ -93,6 +117,20 @@ swmd::SReac::~SReac(void)
 
 ////////////////////////////////////////////////////////////////////////////////
 
+void swmd::SReac::checkpoint(std::fstream & cp_file)
+{
+    cp_file.write((char*)&pCcst, sizeof(double));
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
+void swmd::SReac::restore(std::fstream & cp_file)
+{
+    cp_file.read((char*)&pCcst, sizeof(double));
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
 bool swmd::SReac::active(void) const
 {
 	uint lsridx = pPatch->def()->sreacG2L(defsr()->gidx());
@@ -250,23 +288,33 @@ void swmd::SReac::reset(void)
 
 void swmd::SReac::resetCcst(void)
 {
-	double vol;
-	if (defsr()->inside() == true)
+	uint lsridx = pPatch->def()->sreacG2L(defsr()->gidx());
+	double kcst = pPatch->def()->kcst(lsridx);
+
+	if (defsr()->surf_surf() == false)
 	{
-		assert(pPatch->iComp() != 0);
-		vol = pPatch->iComp()->def()->vol();
+		double vol;
+		if (defsr()->inside() == true)
+		{
+			assert(pPatch->iComp() != 0);
+			vol = pPatch->iComp()->def()->vol();
+		}
+		else
+		{
+			assert (pPatch->oComp() != 0);
+			vol = pPatch->oComp()->def()->vol();
+		}
+
+		pCcst = comp_ccst_vol(kcst, vol, defsr()->order());
 	}
 	else
 	{
-		assert (pPatch->oComp() != 0);
-		vol = pPatch->oComp()->def()->vol();
+		double area;
+		area = pPatch->def()->area();
+		pCcst = comp_ccst_area(kcst, area, defsr()->order());
 	}
 
-	uint lsridx = pPatch->def()->sreacG2L(pSReacdef->gidx());
-	double kcst = pPatch->def()->kcst(lsridx);
-	pCcst = comp_ccst(kcst, vol, pSReacdef->order());
 	assert (pCcst >= 0);
-
 }
 
 ////////////////////////////////////////////////////////////////////////////////
