@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2009Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -24,9 +24,9 @@
 
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 // Standard library & STL headers.
@@ -36,6 +36,7 @@
 #include <iostream>
 #include <sstream>
 #include <algorithm>
+#include <limits>
 
 // STEPS headers.
 #include "../common.h"
@@ -140,6 +141,48 @@ swmd::Wmdirect::~Wmdirect(void)
     delete[] pRannum;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+
+void swmd::Wmdirect::checkpoint(std::string const & file_name)
+{
+	std::fstream cp_file;
+
+    cp_file.open(file_name.c_str(),
+                std::fstream::out | std::fstream::binary | std::fstream::trunc);
+
+    CompPVecCI comp_e = pComps.end();
+    for (CompPVecCI c = pComps.begin(); c != comp_e; ++c) (*c)->checkpoint(cp_file);
+    PatchPVecCI patch_e = pPatches.end();
+    for (PatchPVecCI p = pPatches.begin(); p != patch_e; ++p) (*p)->checkpoint(cp_file);
+
+	statedef()->checkpoint(cp_file);
+
+    cp_file.close();
+}
+
+///////////////////////////////////////////////////////////////////////////////
+
+void swmd::Wmdirect::restore(std::string const & file_name)
+{
+	std::fstream cp_file;
+
+    cp_file.open(file_name.c_str(),
+                std::fstream::in | std::fstream::binary);
+
+    cp_file.seekg(0);
+
+    CompPVecCI comp_e = pComps.end();
+    for (CompPVecCI c = pComps.begin(); c != comp_e; ++c) (*c)->restore(cp_file);
+    PatchPVecCI patch_e = pPatches.end();
+    for (PatchPVecCI p = pPatches.begin(); p != patch_e; ++p) (*p)->restore(cp_file);
+
+	statedef()->restore(cp_file);
+
+    cp_file.close();
+
+    _reset();
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 
 uint swmd::Wmdirect::_addComp(steps::solver::Compdef * cdef)
@@ -368,7 +411,13 @@ double swmd::Wmdirect::_getCompCount(uint cidx, uint sidx) const
 	ssolver::Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint slidx = comp->specG2L(sidx);
-	if (slidx == ssolver::LIDX_UNDEFINED) return 0.0;
+	if (slidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
+
 	return comp->pools()[slidx];
 }
 
@@ -381,7 +430,20 @@ void swmd::Wmdirect::_setCompCount(uint cidx, uint sidx, double n)
 	ssolver::Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint slidx = comp->specG2L(sidx);
-	if (slidx == ssolver::LIDX_UNDEFINED) return;
+	if (slidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
+
+	if (n > std::numeric_limits<unsigned int>::max( ))
+	{
+		std::ostringstream os;
+		os << "Can't set count greater than maximum unsigned integer (";
+		os << std::numeric_limits<unsigned int>::max( ) << ").\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	assert (n >= 0.0);
 	double n_int = std::floor(n);
@@ -451,7 +513,12 @@ bool swmd::Wmdirect::_getCompClamped(uint cidx, uint sidx) const
 	ssolver::Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lsidx = comp->specG2L(sidx);
-    if (lsidx == ssolver::LIDX_UNDEFINED) return false;
+    if (lsidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
     return comp->clamped(lsidx);
 }
@@ -465,7 +532,12 @@ void swmd::Wmdirect::_setCompClamped(uint cidx, uint sidx, bool b)
 	ssolver::Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lsidx = comp->specG2L(sidx);
-    if (lsidx == ssolver::LIDX_UNDEFINED) return;
+    if (lsidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
     comp->setClamped(lsidx, b);
 
@@ -480,7 +552,12 @@ double swmd::Wmdirect::_getCompReacK(uint cidx, uint ridx) const
 	ssolver::Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lridx = comp->reacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return 0.0;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Reaction undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	return comp->kcst(lridx);
 }
@@ -495,7 +572,12 @@ void swmd::Wmdirect::_setCompReacK(uint cidx, uint ridx, double kf)
 	ssolver::Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lridx = comp->reacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Reaction undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	comp->setKcst(lridx, kf);
 
@@ -520,7 +602,12 @@ bool swmd::Wmdirect::_getCompReacActive(uint cidx, uint ridx) const
 	ssolver::Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lridx = comp->reacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return false;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Reaction undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	return (comp->active(lridx));
 }
@@ -534,7 +621,12 @@ void swmd::Wmdirect::_setCompReacActive(uint cidx, uint ridx, bool a)
 	ssolver::Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lridx = comp->reacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Reaction undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	comp->setActive(lridx, a);
 
@@ -551,7 +643,12 @@ double swmd::Wmdirect::_getCompReacC(uint cidx, uint ridx) const
 	ssolver::Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lridx = comp->reacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return 0.0;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Reaction undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	// The 'local' Comp object has same index as solver::Compdef object
 	swmd::Comp * lcomp = pComps[cidx];
@@ -573,7 +670,12 @@ double swmd::Wmdirect::_getCompReacH(uint cidx, uint ridx) const
 	ssolver::Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lridx = comp->reacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return 0.0;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Reaction undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	// The 'local' Comp object has same index as solver::Compdef object
 	swmd::Comp * lcomp = pComps[cidx];
@@ -595,7 +697,12 @@ double swmd::Wmdirect::_getCompReacA(uint cidx, uint ridx) const
 	ssolver::Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lridx = comp->reacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return 0.0;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Reaction undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	// The 'local' Comp object has same index as solver::Compdef object
 	swmd::Comp * lcomp = pComps[cidx];
@@ -617,7 +724,12 @@ uint swmd::Wmdirect::_getCompReacExtent(uint cidx, uint ridx) const
 	ssolver::Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lridx = comp->reacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return 0;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Reaction undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	// The 'local' Comp object has same index as solver::Compdef object
 	swmd::Comp * lcomp = pComps[cidx];
@@ -639,7 +751,12 @@ void swmd::Wmdirect::_resetCompReacExtent(uint cidx, uint ridx)
 	ssolver::Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lridx = comp->reacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Reaction undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	// The 'local' Comp object has same index as solver::Compdef object
 	swmd::Comp * lcomp = pComps[cidx];
@@ -681,7 +798,12 @@ double swmd::Wmdirect::_getPatchCount(uint pidx, uint sidx) const
 	ssolver::Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint slidx = patch->specG2L(sidx);
-	if (slidx == ssolver::LIDX_UNDEFINED) return 0.0;
+	if (slidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	return patch->pools()[slidx];
 }
@@ -695,7 +817,19 @@ void swmd::Wmdirect::_setPatchCount(uint pidx, uint sidx, double n)
 	ssolver::Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint slidx = patch->specG2L(sidx);
-	if (slidx == ssolver::LIDX_UNDEFINED) return;
+	if (slidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
+	if (n > std::numeric_limits<unsigned int>::max( ))
+	{
+		std::ostringstream os;
+		os << "Can't set count greater than maximum unsigned integer (";
+		os << std::numeric_limits<unsigned int>::max( ) << ").\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	double n_int = std::floor(n);
 	double n_frc = n-n_int;
@@ -723,7 +857,7 @@ double swmd::Wmdirect::_getPatchAmount(uint pidx, uint sidx) const
 
 void swmd::Wmdirect::_setPatchAmount(uint pidx, uint sidx, double a)
 {
-	assert(a > 0.0);
+	assert(a >= 0.0);
 	// convert amount in mols to number of molecules
 	double a2 = a * steps::math::AVOGADRO;
 	// the following method does all the necessary argument checking
@@ -739,7 +873,12 @@ bool swmd::Wmdirect::_getPatchClamped(uint pidx, uint sidx) const
 	ssolver::Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lsidx = patch->specG2L(sidx);
-    if (lsidx == ssolver::LIDX_UNDEFINED) return false;
+    if (lsidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
     return patch->clamped(lsidx);
 }
@@ -753,7 +892,12 @@ void swmd::Wmdirect::_setPatchClamped(uint pidx, uint sidx, bool buf)
 	ssolver::Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lsidx = patch->specG2L(sidx);
-    if (lsidx == ssolver::LIDX_UNDEFINED) return;
+    if (lsidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
     patch->setClamped(lsidx, buf);
 
@@ -768,7 +912,12 @@ double swmd::Wmdirect::_getPatchSReacK(uint pidx, uint ridx) const
 	ssolver::Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lridx = patch->sreacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return 0.0;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Surface reaction undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	return patch->kcst(lridx);
 }
@@ -783,7 +932,12 @@ void swmd::Wmdirect::_setPatchSReacK(uint pidx, uint ridx, double kf)
 	ssolver::Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lridx = patch->sreacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Surface reaction undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	patch->setKcst(lridx, kf);
 
@@ -809,7 +963,12 @@ bool swmd::Wmdirect::_getPatchSReacActive(uint pidx, uint ridx) const
 	ssolver::Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lridx = patch->sreacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return false;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Surface reaction undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	return (patch->active(lridx));
 }
@@ -823,7 +982,12 @@ void swmd::Wmdirect::_setPatchSReacActive(uint pidx, uint ridx, bool a)
 	ssolver::Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lridx = patch->sreacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Surface reaction undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	patch->setActive(lridx, a);
 
@@ -841,7 +1005,12 @@ double swmd::Wmdirect::_getPatchSReacC(uint pidx, uint ridx) const
 	ssolver::Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lridx = patch->sreacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return 0.0;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Surface reaction undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	// The 'local' Patch object has same index as solver::Patchdef object
 	swmd::Patch * lpatch = pPatches[pidx];
@@ -864,7 +1033,12 @@ double swmd::Wmdirect::_getPatchSReacH(uint pidx, uint ridx) const
 	ssolver::Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lridx = patch->sreacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return 0.0;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Surface reaction undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	// The 'local' Patch object has same index as solver::Patchdef object
 	swmd::Patch * lpatch = pPatches[pidx];
@@ -887,7 +1061,12 @@ double swmd::Wmdirect::_getPatchSReacA(uint pidx, uint ridx) const
 	ssolver::Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lridx = patch->sreacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return 0.0;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Surface reaction undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	// The 'local' Patch object has same index as solver::Patchdef object
 	swmd::Patch * lpatch = pPatches[pidx];
@@ -910,7 +1089,12 @@ uint swmd::Wmdirect::_getPatchSReacExtent(uint pidx, uint ridx) const
 	ssolver::Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lridx = patch->sreacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return 0;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Surface reaction undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	// The 'local' Patch object has same index as solver::Patchdef object
 	swmd::Patch * lpatch = pPatches[pidx];
@@ -933,7 +1117,12 @@ void swmd::Wmdirect::_resetPatchSReacExtent(uint pidx, uint ridx)
 	ssolver::Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lridx = patch->sreacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Surface reaction undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	// The 'local' Patch object has same index as solver::Patchdef object
 	swmd::Patch * lpatch = pPatches[pidx];
@@ -1235,4 +1424,3 @@ void swmd::Wmdirect::_executeStep(swmd::KProc * kp, double dt)
 
 // END
 
-
