@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2009Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -23,9 +23,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 #ifndef STEPS_SOLVER_WMDIRECT_HPP
@@ -37,6 +37,7 @@
 #include <vector>
 #include <set>
 #include <map>
+#include <fstream>
 
 // STEPS headers.
 #include "../common.h"
@@ -84,6 +85,15 @@ public:
     Wmdirect(steps::model::Model * m, steps::wm::Geom * g, steps::rng::RNG * r);
     ~Wmdirect(void);
 
+    ////////////////////////////////////////////////////////////////////////
+    // CHECKPOINTING
+    ////////////////////////////////////////////////////////////////////////
+    /// checkpoint data
+    void checkpoint(std::string const & file_name);
+
+    /// restore data
+    void restore(std::string const & file_name);
+
     ////////////////////////////////////////////////////////////////////////
     // SOLVER INFORMATION
     ////////////////////////////////////////////////////////////////////////
@@ -107,7 +117,7 @@ public:
     // SOLVER STATE ACCESS:
     //      GENERAL
     ////////////////////////////////////////////////////////////////////////
-    
+
 
     double getTime(void) const;
 
@@ -120,11 +130,11 @@ public:
     // SOLVER STATE ACCESS:
     //      ADVANCE
     //      Developer only
-    //////////////////////////////////////////////////////////////////////// 
-    
-    void setTime(double time);    
+    ////////////////////////////////////////////////////////////////////////
+
+    void setTime(double time);
     void setNSteps(uint nsteps);
-    
+
     ////////////////////////////////////////////////////////////////////////
     // SOLVER STATE ACCESS:
     //      COMPARTMENT
