@@ -1,6 +1,6 @@
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 # STEPS - STochastic Engine for Pathway Simulation
-# Copyright (C) 2007-2009 Okinawa Institute of Science and Technology, Japan.
+# Copyright (C) 2007-2011 Okinawa Institute of Science and Technology, Japan.
 # Copyright (C) 2003-2006 University of Antwerp, Belgium.
 #
 # See the file AUTHORS for details.
@@ -22,9 +22,9 @@
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 
-#  Last Changed Rev:  $Rev: 349 $
-#  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
-#  Last Changed By:   $Author: wchen $
+#  Last Changed Rev:  $Rev: 410 $
+#  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+#  Last Changed By:   $Author: iain $
 
 """ 
 .. Note::
@@ -245,6 +245,12 @@ class ElementProxy:
         """
 
         self.groups[group_name] = group_ids
+        
+    def addToGroup(self, group_name, id):
+        if (group_name in self.groups):
+            self.groups[group_name].append(id)
+        else:
+            self.groups[group_name] = [id]
 
     def getGroups(self):
         """
@@ -496,7 +502,8 @@ def importTetGen(pathroot):
         tet[2] = nodeproxy.getSTEPSID(int(tokens[3]))
         tet[3] = nodeproxy.getSTEPSID(int(tokens[4]))
         tetproxy.insert(tetid, tet)
-    # Close the file.
+        if nattribs == 1:
+            tetproxy.addToGroup(tokens[5], tetproxy.getSTEPSID(tetid))
     elefile.close()
 
     # Try to read the tri file.
@@ -526,8 +533,10 @@ def importTetGen(pathroot):
             tri[1] = nodeproxy.getSTEPSID(int(tokens[2]))
             tri[2] = nodeproxy.getSTEPSID(int(tokens[3]))
             triproxy.insert(triid, tri)
-        # Close the file.
-        facefile.close()
+            if nattribs == 1:
+                triproxy.addToGroup(tokens[4], triproxy.getSTEPSID(triid))
+    # Close the file.
+    facefile.close()
 
     print("Read TetGen files succesfully")    
     
@@ -547,7 +556,7 @@ def importTetGen(pathroot):
 
 #############################################################################################
 
-def importAbaqus(filename, scale):
+def importAbaqus(filename, scale, ebs = None):
     """ 
     Read a ABAQUS-formated mesh file, return the created steps.geom.Tetmesh object, 
     the element mapping for nodes, tetraedrons and triangles.
@@ -557,6 +566,7 @@ def importAbaqus(filename, scale):
     * filename: the Abaqus filename (or path) including any suffix.
     * scale: LENGTH scale from the importing mesh to real geometry. e.g. a radius 
       of 10 in the importing file to a radius of 1 micron in STEPS, scale is 1e-7.
+    * ebs: specify the names of selected element blocks which are included in the mesh
     
     RETURNS:
     
@@ -589,6 +599,10 @@ def importAbaqus(filename, scale):
     # check we have the right kind of CUBIT output file here
     assert(line == '*HEADING\n')
     
+    if ebs == None:
+        include = True
+    else:
+        include = False
     currmap = None
     # read line one by one until the end
     while(line):
@@ -598,7 +612,7 @@ def importAbaqus(filename, scale):
         elif (line.find('*NODE', 0, 5) == 0):
             if (currmap != None):
                 currmap.blockEnd()
-            print('Found *NODE section, start reading nodes.')
+            #print('Found *NODE section, start reading nodes.')
             lineset = line.split('=') 
             if (len(lineset) == 1):
                 nodeproxy.blockBegin('AllNodes')
@@ -608,16 +622,20 @@ def importAbaqus(filename, scale):
         elif (line.find('*ELEMENT', 0, 8) == 0):
             if (currmap != None):
                 currmap.blockEnd()
-            print('Found *Element section, start reading elements.')
             lineset = line.split(',') 
             elementtype = lineset[1].split('=')[1]
             blockname = 'AllElements'
             if (len(lineset) == 3):
                 blockname = lineset[2].split('=')[1].strip('\n')
-            if (elementtype == 'C3D4'):
+            if (ebs == None) or (blockname in ebs):
+                include = True
+                #print('Importing *Element section: %s.' % (blockname))
+            else:
+                include  = False
+            if (elementtype == 'C3D4') and include:
                 currmap = tetproxy
                 tetproxy.blockBegin(blockname)
-            elif (elementtype == 'STRI3'):
+            elif (elementtype == 'STRI3') and include:
                 currmap = triproxy
                 triproxy.blockBegin(blockname)
         elif (line.find('**', 0, 2) == -1 and line[0] == '*'):
@@ -635,7 +653,7 @@ def importAbaqus(filename, scale):
                 node[1] = float(linesec[2])*scale
                 node[2] = float(linesec[3])*scale 
                 currmap.insert(nodeid, node)
-            elif (currmap == tetproxy): 
+            elif (currmap == tetproxy) and include: 
                 linesec = line.replace(',', ' ')
                 linesec = linesec.split() 
                 nodeid = int(linesec[0])
@@ -646,7 +664,7 @@ def importAbaqus(filename, scale):
                 node[2] = nodeproxy.getSTEPSID(int(linesec[3])) 
                 node[3] = nodeproxy.getSTEPSID(int(linesec[4])) 
                 currmap.insert(nodeid, node)
-            elif (currmap == triproxy):
+            elif (currmap == triproxy) and include:
                 linesec = line.replace(',', ' ')
                 linesec = linesec.split() 
                 nodeid = int(linesec[0])
@@ -675,6 +693,118 @@ def importAbaqus(filename, scale):
     
     return mesh, nodeproxy, tetproxy,triproxy
 
+#############################################################################################
+
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
+
+#############################################################################################
+
+def importGmsh(filename, scale):
+    """ 
+    Read a Gmsh-formated mesh file, return the created steps.geom.Tetmesh object, 
+    the element mapping for nodes, tetraedrons and triangles.
+    
+    PARAMETERS:
+    
+    * filename: the Abaqus filename (or path) including any suffix.
+    * scale: LENGTH scale from the importing mesh to real geometry. e.g. a radius 
+      of 10 in the importing file to a radius of 1 micron in STEPS, scale is 1e-7.
+    
+    RETURNS:
+    
+    mesh, nodeproxy, tetproxy, triproxy 
+
+    * mesh: The STEPS TetMesh object
+    * nodeproxy: Element Map for nodes
+    * tetproxy: Element Map for tetrahedrons
+    * triproxy: Element Map for triangles
+
+   IMPORTANT NOTICE:
+   
+   User is recommanded to save the tetmesh objects using the saveTetmesh() method
+   and recreate the objects from the saved files, instead of creating the objects
+   via the importing functions each time, if the tetmesh objects are intented to
+   be used in multiple simulations. Since the importing functions require a massive
+   amount of time to create the Tetmesh object, comparing to the loadTetmesh() method.
+
+    """
+    print("Reading Gmsh file...")
+    btime = time.time()
+    
+    meshfile = open(filename, 'r')
+    
+    nodeproxy = ElementProxy('node', 3)
+    tetproxy = ElementProxy('tet', 4)
+    triproxy = ElementProxy('tri', 3)
+
+    line = meshfile.readline()
+    assert(line == '$MeshFormat\n')
+    line = meshfile.readline()
+    line = meshfile.readline()
+    assert(line == '$EndMeshFormat\n')
+    line = meshfile.readline()
+    assert(line == '$Nodes\n')
+    line = meshfile.readline()
+    n_nodes = int(line)
+    # read nodes
+    for i in range(n_nodes):
+        line = meshfile.readline()
+        linesec = line.split()
+        id = int(linesec[0])
+        node = [0.0,0.0,0.0]
+        node[0] = float(linesec[1])*scale
+        node[1] = float(linesec[2])*scale
+        node[2] = float(linesec[3])*scale
+        nodeproxy.insert(id, node)
+    line = meshfile.readline()
+    assert(line == '$EndNodes\n')   
+    
+    line = meshfile.readline()
+    assert(line == '$Elements\n')
+    
+    line = meshfile.readline()
+    n_elems = int(line)
+    # read elem
+    for i in range(n_elems):
+        line = meshfile.readline()
+        linesec = line.split()
+        id = int(linesec[0])
+        type = int(linesec[1])
+        # triangle
+        if type == 2:
+            node = [0,0,0]
+            node[0] = nodeproxy.getSTEPSID(int(linesec[6]))
+            node[1] = nodeproxy.getSTEPSID(int(linesec[7]))
+            node[2] = nodeproxy.getSTEPSID(int(linesec[8])) 
+            triproxy.insert(id, node)
+        # tet
+        if type == 4:
+            group_id = linesec[4]
+            node = [0,0,0,0]
+            node[0] = nodeproxy.getSTEPSID(int(linesec[6]))
+            node[1] = nodeproxy.getSTEPSID(int(linesec[7]))
+            node[2] = nodeproxy.getSTEPSID(int(linesec[8])) 
+            node[3] = nodeproxy.getSTEPSID(int(linesec[9])) 
+            tetproxy.insert(id, node)
+            tetproxy.addToGroup(group_id, tetproxy.getSTEPSID(id))
+            
+    line = meshfile.readline()
+    assert(line == '$EndElements\n')
+    meshfile.close()
+    print("Read Msh file succesfully")    
+    
+    nodedata = nodeproxy.getAllData()
+    tetdata = tetproxy.getAllData() 
+    tridata = triproxy.getAllData() 
+
+    print("creating Tetmesh object in STEPS...")
+    mesh = stetmesh.Tetmesh(nodedata, tetdata, tridata)
+    print("Tetmesh object created.")
+    
+    return mesh, nodeproxy, tetproxy,triproxy
+
+
+
 #############################################################################################
 
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -1113,6 +1243,4 @@ def loadMesh(pathname):
 
     
     return (mesh,comps_out,patches_out)
-#############################################################################################
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
